@@ -566,6 +566,29 @@ static inline void handle_bragi_key_input(unsigned char* kbinput, const unsigned
     }
 }
 
+// We just need the first few buttons only, so realistically it doesn't matter
+#define BUTTON_BLD_COUNT 5
+
+static inline void hid_mouse_bld_translate(usbinput* input, int length, const unsigned char* urbinput){
+    if(length < 4){
+        ckb_err("Invalid length %d", length);
+        return;
+    }
+    // Byte 0 = mouse buttons (bitfield)
+    for(int bit = 0; bit < BUTTON_BLD_COUNT; bit++){
+        if(urbinput[0] & (1 << bit))
+            SET_KEYBIT(input->keys, MOUSE_BUTTON_FIRST + bit);
+        else
+            CLEAR_KEYBIT(input->keys, MOUSE_BUTTON_FIRST + bit);
+    }
+    // Bytes 1 - 2: movement
+    input->rel_x += (signed char)urbinput[1];
+    input->rel_y += (signed char)urbinput[2];
+    // Byte 3: wheel
+    input->whl_rel_y = (signed char)urbinput[3];
+}
+
+
 void process_input_urb(void* context, unsigned char* buffer, int urblen, ushort ep){
     if(!urblen)
         return;
@@ -617,6 +640,9 @@ void process_input_urb(void* context, unsigned char* buffer, int urblen, ushort
 
     // If the response starts with CMD_GET (0x0e) for NXP, or it came from a bragi command EP, that means it needs to go to os_usbrecv()
     if(urblen == kb->out_ep_packet_size && (firstbyte == CMD_GET || (kb->protocol == PROTO_BRAGI && ep == kb->bragi_in_ep))){
+#ifdef DEBUG_USB_RECV
+    print_urb_buffer("Recv:", buffer, urblen, NULL, 0, NULL, INDEX_OF(targetkb, keyboard), (uchar)ep);
+#endif
         int retval = pthread_mutex_lock(intmutex(targetkb));
         if(retval)
             ckb_fatal("Error locking interrupt mutex %i", retval);
@@ -666,7 +692,12 @@ void process_input_urb(void* context, unsigned char* buffer, int urblen, ushort
                         hid_mouse_translate(&targetkb->input, urblen, buffer);
                     }
                 } else if(firstbyte == MOUSE_IN) {
-                    hid_mouse_translate(&kb->input, urblen, buffer + 1);
+                    // If we're in bootloader mode, parse the simplified reports
+                    // This is only tested against an M65 with bld version 3
+                    if(kb->needs_fw_update)
+                        hid_mouse_bld_translate(&kb->input, urblen - 1, buffer + 1);
+                    else
+                        hid_mouse_translate(&kb->input, urblen - 1, buffer + 1);
                 } else if(firstbyte == CORSAIR_IN) { // Corsair Mouse Input
                     corsair_mousecopy(kb->input.keys, buffer);
                 } else if(firstbyte == 0x05 && urblen == 21) { // Seems to be on the Ironclaw RGB only
@@ -731,14 +762,32 @@ void process_input_urb(void* context, unsigned char* buffer, int urblen, ushort
                         ckb_err("Unknown bragi data received in input thread %02x from endpoint %02x", firstbyte, ep);
                     }
                 } else {
-                    // Accept NKRO only if device is not active
+#define NXP_STRAFE_MEDIA_WORKAROUND(kb) (kb->vendor == V_CORSAIR && kb->product == P_STRAFE)
+#define NXP_STRAFE_MEDIA_MASK 0x3e
+//#define NXP_STRAFE_VOL_MASK 0xc
+                    // Accept NKRO only if device is not active, unless it's media keys and it's an original strafe with the media key bug
                     if(firstbyte == NKRO_KEY_IN || firstbyte == NKRO_MEDIA_IN) {
-                        if(!targetkb->active)
+                        if(!targetkb->active || (firstbyte == NKRO_MEDIA_IN && NXP_STRAFE_MEDIA_WORKAROUND(targetkb)))
                             hid_kb_translate(targetkb->input.keys, urblen, buffer, 0);
                     } else if(urblen == MSG_SIZE) {
-                        if((kb->fwversion >= 0x130 || IS_V2_OVERRIDE(kb)) && firstbyte == CORSAIR_IN) // Ugly hack due to old FW 1.15 packets having no header
+                        if((kb->fwversion >= 0x130 || IS_V2_OVERRIDE(kb)) && firstbyte == CORSAIR_IN) // Ugly hack due to old FW 1.15 packets having no report id
                             buffer++;
-                        corsair_kbcopy(targetkb->input.keys, buffer);
+                        // If it's an original strafe RGB, to work around the media keys getting stuck down on the 03 packet,
+                        // we save the bits before the kbcopy and then restore them afterwards for the 02 packet to take effect.
+                        if(NXP_STRAFE_MEDIA_WORKAROUND(targetkb)) {
+                            // Store
+                            uchar media_key_state_1 = targetkb->input.keys[12];
+                            uchar media_key_state_2 = targetkb->input.keys[16];
+                            // Apply fresh key data
+                            corsair_kbcopy(targetkb->input.keys, buffer);
+                            // Restore
+                            targetkb->input.keys[12] = (targetkb->input.keys[12] & ~NXP_STRAFE_MEDIA_MASK) | (media_key_state_1 & NXP_STRAFE_MEDIA_MASK);
+                            //targetkb->input.keys[16] = (targetkb->input.keys[16] & ~NXP_STRAFE_VOL_MASK) | (media_key_state_2 & NXP_STRAFE_VOL_MASK);
+                            // We can just copy this byte as-is, as there are no M or G keys in the strafe.
+                            targetkb->input.keys[16] = media_key_state_2;
+                        } else {
+                            corsair_kbcopy(targetkb->input.keys, buffer);
+                        }
                     } else {
                         ckb_err("Unknown data received in input thread %02x from endpoint %02x", firstbyte, ep);
                     }
@@ -756,7 +805,7 @@ void handle_nkro_key_input(unsigned char* kbinput, const unsigned char* urbinput
     int bytelen = (legacy ? 14 : 19);
     int start = !legacy; // Legacy packets start from 0x00, other ones start from 0x01
     if(length < start + bytelen + 1){
-        ckb_err("Invalid length %d in handle_nkro_key_input() legacy %d", length, legacy);
+        ckb_err("Invalid length %d legacy %d", length, legacy);
         return;
     }
 
@@ -876,7 +925,11 @@ void hid_kb_translate(unsigned char* kbinput, int length, const unsigned char* u
 #define BUTTON_HID_COUNT    5
 
 void hid_mouse_translate(usbinput* input, int length, const unsigned char* urbinput){
-    // Byte 1 = mouse buttons (bitfield)
+    if(length < 9){
+        ckb_err("Invalid length %d", length);
+        return;
+    }
+    // Byte 0 = mouse buttons (bitfield)
     for(int bit = 0; bit < BUTTON_HID_COUNT; bit++){
         if(urbinput[0] & (1 << bit))
             SET_KEYBIT(input->keys, MOUSE_BUTTON_FIRST + bit);
@@ -887,7 +940,7 @@ void hid_mouse_translate(usbinput* input, int length, const unsigned char* urbin
     input->rel_x += (urbinput[5] << 8) | urbinput[4];
     input->rel_y += (urbinput[7] << 8) | urbinput[6];
     // Byte 9: wheel
-    input->whl_rel_y = (char)urbinput[8];
+    input->whl_rel_y = (signed char)urbinput[8];
 }
 
 void corsair_mousecopy(unsigned char* kbinput, const unsigned char* urbinput){
@@ -932,7 +985,7 @@ void m95_mouse_translate(usbinput* kbinput, int length, const unsigned char* urb
 
     kbinput->rel_x += (urbinput[3] << 8) | urbinput[2];
     kbinput->rel_y += (urbinput[5] << 8) | urbinput[4];
-    kbinput->whl_rel_y = (char)urbinput[6];
+    kbinput->whl_rel_y = (signed char)urbinput[6];
 }
 
 #define BRAGI_MOUSE_BUTTONS 16
@@ -1003,7 +1056,7 @@ void corsair_bragi_mousecopy(usbdevice* kb, usbinput* input, const unsigned char
 
     // Some devices only have one byte, so set those to 8 buttons
     // We need a better way to identify this
-    if(kb->vendor == V_CORSAIR && (kb->product == P_M55_RGB_PRO || kb->product == P_DARK_CORE_RGB_PRO_SE || kb->product == P_DARK_CORE_RGB_PRO_SE_WL || kb->product == P_HARPOON_WL_U))
+    if(kb->vendor == V_CORSAIR && (kb->product == P_M55_RGB_PRO || kb->product == P_DARK_CORE_RGB_PRO_SE || kb->product == P_DARK_CORE_RGB_PRO_SE_WL || kb->product == P_HARPOON_WL_U || kb->product == P_DARK_CORE_RGB_PRO || kb->product == P_DARK_CORE_RGB_PRO_WL))
         buttons = BRAGI_ONE_BYTE_MOUSE_BUTTONS;
 
     // Pick the appropriate LUT. We can't patch the keymap as that will break standard HID input.
