@@ -36,6 +36,8 @@ const dpi_list mouse_dpi_list[] = {
     { P_NIGHTSWORD, 18000},
     { P_IRONCLAW_W_U, 18000 },
     { P_HARPOON_WL_U, 10000 },
+    { P_DARK_CORE_RGB_PRO, 18000 },
+    { P_DARK_CORE_RGB_PRO_SE, 18000 },
     { P_GLAIVE_PRO, 16000 },
     { 0, 0 }, // Keep last and do not remove
 };
@@ -44,9 +46,13 @@ const dpi_list mouse_dpi_list[] = {
 const device_desc models[] = {
     // Keyboards
     { V_CORSAIR, P_K55, },
+    { V_CORSAIR, P_K55_PRO, },
+    { V_CORSAIR, P_K55_PRO_XT, },
     { V_CORSAIR, P_K60_PRO_RGB, },
     { V_CORSAIR, P_K60_PRO_RGB_LP, },
     { V_CORSAIR, P_K60_PRO_RGB_SE, },
+    { V_CORSAIR, P_K60_PRO_MONO, },
+    { V_CORSAIR, P_K60_PRO_TKL, },
     { V_CORSAIR, P_K63_NRGB, },
     { V_CORSAIR, P_K63_NRGB_WL, },
     { V_CORSAIR, P_K63_NRGB_WL2, },
@@ -104,6 +110,8 @@ const device_desc models[] = {
     { V_CORSAIR, P_DARK_CORE_SE, },
     { V_CORSAIR, P_DARK_CORE_WL, },
     { V_CORSAIR, P_DARK_CORE_SE_WL, },
+    { V_CORSAIR, P_DARK_CORE_RGB_PRO, },
+    { V_CORSAIR, P_DARK_CORE_RGB_PRO_WL, },
     { V_CORSAIR, P_DARK_CORE_RGB_PRO_SE, },
     { V_CORSAIR, P_DARK_CORE_RGB_PRO_SE_WL, },
     { V_CORSAIR, P_IRONCLAW_W_U, },
@@ -115,6 +123,8 @@ const device_desc models[] = {
     { V_CORSAIR, P_POLARIS, },
     // Headset stands
     { V_CORSAIR, P_ST100, },
+    // Misc
+    { V_CORSAIR, P_GENERIC_BRAGI_DONGLE, },
 };
 
 const size_t N_MODELS = sizeof(models) / sizeof(device_desc);
@@ -196,12 +206,16 @@ const char* product_str(ushort product){
         return "k63";
     if(product == P_K63_NRGB_WL || product == P_K63_NRGB_WL2 || product == P_K63_NRGB_WL3 || product == P_K63_NRGB_WL4)
         return "k63_wireless";
-    if(product == P_K60_PRO_RGB || product == P_K60_PRO_RGB_LP || product == P_K60_PRO_RGB_SE)
+    if(product == P_K60_PRO_RGB || product == P_K60_PRO_RGB_LP || product == P_K60_PRO_RGB_SE || product == P_K60_PRO_MONO)
         return "k60";
-    if(product == P_K57_U || product == P_K57_D)
+    if(product == P_K60_PRO_TKL)
+        return "k60_tkl";
+    if(product == P_K57_U || product == P_K57_D || product == P_K55_PRO_XT)
         return "k57_wireless";
     if(product == P_K55)
         return "k55";
+    if(product == P_K55_PRO)
+        return "k55pro";
     if(product == P_STRAFE || product == P_STRAFE_NRGB || product == P_STRAFE_NRGB_2)
         return "strafe";
     if(product == P_STRAFE_MK2)
@@ -233,14 +247,18 @@ const char* product_str(ushort product){
         return "ironclaw";
     if(product == P_NIGHTSWORD)
         return "nightsword";
-    if(product == P_IRONCLAW_W_U || product == P_IRONCLAW_W_D || product == P_DARK_CORE_RGB_PRO_SE || product == P_DARK_CORE_RGB_PRO_SE_WL || product == P_HARPOON_WL_U || product == P_HARPOON_WL_D)
+    if(product == P_IRONCLAW_W_U || product == P_IRONCLAW_W_D || product == P_HARPOON_WL_U || product == P_HARPOON_WL_D)
         return "ironclaw_wireless";
+    if(product == P_DARK_CORE_RGB_PRO || product == P_DARK_CORE_RGB_PRO_WL || product == P_DARK_CORE_RGB_PRO_SE || product == P_DARK_CORE_RGB_PRO_SE_WL)
+        return "dark_core_rgb_pro";
     if(product == P_POLARIS)
         return "polaris";
     if(product == P_DARK_CORE || product == P_DARK_CORE_WL || product == P_DARK_CORE_SE || product == P_DARK_CORE_SE_WL)
         return "darkcore";
     if(product == P_ST100)
         return "st100";
+    if(product == P_GENERIC_BRAGI_DONGLE)
+        return "bragi_dongle";
     return "";
 }
 
@@ -288,84 +306,52 @@ static const devcmd* get_vtable(usbdevice* kb){
 }
 
 // USB device main loop
-/// brief .
-///
-/// \brief devmain is called by _setupusb
-/// \param kb the pointer to the device. Even if it has the name kb, it is valid also for a mouse (the whole driver seems to be implemented first for a keyboard).
-/// \return always a nullptr
-///
-/// # Synchronization
-/// The syncing via mutexes is interesting:
-/// 1. \a imutex (the Input mutex)\n
-/// This one is locked in \c setupusb().
-/// That function does only two things: Locking the mutex and trying to start a thread at \c _setupusb().
-/// _setupusb() unlocks \a imutex  after getting some buffers and initalizing internal structures from the indicators
-/// (this function often gets problems with error messages like "unable to read indicators" or "Timeout bla blubb").
-/// \warning have a look at \a updateindicators() later.
-/// \warning if creating the thread is not successful, the \a imutex remains blocked. Have a look at setupusb() later.
-///
-/// 2. \a dmutex (the Device mutex)\n
-/// This one is very interesting, because it is handled in devmain().
-/// It seems that it is locked only in \a _ledthread(), which is a thread created in \a os_setupindicators().
-/// os_setupindicators() again is called in \a _setupusb() long before calling devmain().
-/// So this mutex is locked when we start the function as the old comment says.\n
-/// Before reading from the FIFO and direct afterwards an unlock..lock sequence is implemented here.
-/// Even if only the function readlines() should be surrounded by the unlock..lock,
-/// the variable definition of the line pointer is also included here. Not nice, but does not bother either.
-/// Probably the Unlock..lock is needed so that now another process can change the control structure \a linectx while we wait in readlines().
-/// \todo Hope to find the need for dmutex usage later.
-/// \n Should this function be declared as pthread_t* function, because of the defintion of pthread-create? But void* works also...
-///
 static void* devmain(usbdevice* kb){
     /// \attention dmutex should still be locked when this is called
-    int kbfifo = kb->infifo - 1;
-    ///
-    /// First a \a readlines_ctx buffer structure is initialized by \c readlines_ctx_init().
-    readlines_ctx linectx;
-    readlines_ctx_init(&linectx);
-    ///
-    /// After some setup functions, beginning in _setupusb() which has called devmain(),
-    /// we read the command input-Fifo designated to that device in an endless loop.
-    /// This loop has two possible exits (plus reaction to signals, not mentioned here).
+    const int kbfifo = kb->infifo - 1;
+    readlines_ctx* linectx = calloc(1, sizeof(readlines_ctx));
+
     while(1){
-        ///
-        /// If the reading via readlines() is successful (we might have read multiple lines),
-        /// the interpretation is done by readcmd() if the connection to the device is still available
-        /// This is true if the kb-structure has a handle and an event pointer both != Null).
-        /// If not, the loop is left (the first exit point).
         queued_mutex_unlock(dmutex(kb));
-        // Read from FIFO
-        const char* line;
-        int lines = readlines(kbfifo, linectx, &line);
+        // Read from cmd FIFO
+        int ret = readline_fifo(kbfifo, linectx);
         wait_until_suspend_processed();
         queued_mutex_lock(dmutex(kb));
+
         // End thread when the handle is removed
         if(kb->status == DEV_STATUS_DISCONNECTING || kb->status == DEV_STATUS_DISCONNECTED)
             break;
-        ///
-        /// if nothing is in the line buffer (some magic interrupt?),
-        /// continue in the endless while without any reaction.
-        if(lines){
-            /// \todo readcmd() gets a \b line, not \b lines. Have a look on that later.
-            if(readcmd(kb, line)){
-                ///
-                /// If interpretation and communication with the usb device got errors,
-                /// they are signalled by readcmd() (non zero retcode).
-                /// In this case the usb device is closed via closeusb()
-                /// and the endless loop is left (the second exit point).
-                // USB transfer failed; destroy device
+
+        if(ret == 0) {
+            // EOF
+            break;
+        } else if(ret < 0) {
+            // Retry
+            continue;
+        } else {
+            if(readcmd(kb, linectx->buf)){
+                // USB transfer failed or command requested disconnect; destroy device
                 closeusb(kb);
-                break;
+                goto cleanup;
             }
         }
     }
+cleanup:
     queued_mutex_unlock(dmutex(kb));
-    ///
-    /// After leaving the endless loop the readlines-ctx structure and its buffers are freed by readlines_ctx_free().
-    readlines_ctx_free(linectx);
+    free(linectx);
     return 0;
 }
 
+void fill_usbdevice_protocol(usbdevice* kb){
+    if(USES_BRAGI(kb->vendor, kb->product))
+        kb->protocol = PROTO_BRAGI;
+
+    if(USES_BRAGI_JUMBO(kb->vendor, kb->product))
+        kb->out_ep_packet_size = BRAGI_JUMBO_SIZE;
+    else
+        kb->out_ep_packet_size = MSG_SIZE;
+}
+
 /// brief .
 ///
 /// \brief _setupusb A horrible function for setting up an usb device
@@ -409,14 +395,7 @@ static void* _setupusb(void* context){
     ///
     usbdevice* kb = context;
     queued_mutex_lock(dmutex(kb));
-    if(USES_BRAGI(kb->vendor, kb->product))
-        kb->protocol = PROTO_BRAGI;
-
-    if(USES_BRAGI_JUMBO(kb->vendor, kb->product))
-        kb->out_ep_packet_size = BRAGI_JUMBO_SIZE;
-    else
-        kb->out_ep_packet_size = MSG_SIZE;
-
+    fill_usbdevice_protocol(kb);
     queued_mutex_lock(imutex(kb));
     // Set standard fields
     ushort vendor = kb->vendor, product = kb->product;
@@ -441,16 +420,11 @@ static void* _setupusb(void* context){
     if(kb->protocol == PROTO_BRAGI)
         kb->features &= ~FEAT_FWUPDATE;
 
-    kb->usbdelay = USB_DELAY_DEFAULT;
-
     // Check if the device needs a patched keymap, and if so patch it.
     patchkeys(kb);
 
     // Perform OS-specific setup
-    ///
-    /// - A fixed 100ms wait is the start.
-    /// <b>Although the DELAY_LONG macro is given a parameter, it is ignored. Occasionally refactor it.</b>
-    DELAY_LONG(kb);
+    DELAY_100MS();
 
     ///
     /// - The first relevant point is the operating system-specific opening of the interface in os_setupusb().
@@ -531,39 +505,7 @@ static void* _setupusb(void* context){
     /// which first performs an unlock on the imutex.
     /// This is interesting because the next statement is exactly this: An unlock on the imutex.
     queued_mutex_unlock(imutex(kb));
-    ///
-    /// - Via vtable the \a kb->start() function is called next.
-    /// This is the same for a mouse and an RGB keyboard: start_dev(),
-    /// for a non RGB keyboard it is start_kb_nrgb().
-    /// \n First parameter is as always kb, second is 0 (makeactive = false).
-    ///   - In start_kb_nrgb() set the keyboard into a so-called software mode (NK95_HWOFF)
-    /// via ioctl with \c usbdevfs_ctrltransfer in function _nk95cmd(),
-    /// which will in turn is called via macro nk95cmd() via start_kb_nrgb().
-    /// \n Then two dummy values (active and pollrate) are set in the kb structure and ready.
-    ///   - start_dev() does a bit more - because this function is for both mouse and keyboard.
-    /// start_dev() calls - after setting an extended timeout parameter - _start_dev(). Both are located in device.c.
-    ///   - First, _start_dev() attempts to determine the firmware version of the device,
-    /// but only if two conditions are met:
-    /// hwload-mode is not null (then hw-loading is disabled)
-    /// and the device has the FEAT_HWLOAD feature.
-    /// Then the firmware and the poll rate are fetched via getfwversion().
-    /// \n If hwload_mode is set to "load only once" (==1), then the HWLOAD feature is masked, so that no further reading can take place.
-    ///   - Now check if device needs a firmware update. If so, set it up and leave the function without error.
-    ///   - Else load the hardware profile from device if the hw-pointer is not set and hw-loading is possible and allowed.
-    /// \n Return error if mode == 2 (load always) and loading got an error.
-    /// Else mask the HWLOAD feature, because hwload must be 1 and the error couold be a repeated hw-reading.
-    /// \n <b>Puh, that is real Horror code. It seems to be not faulty, but completely unreadable.</b>
-    ///
-    ///   - Finally, the second parameter of _startdev() is used to check whether the device is to be activated.
-    /// Depending on the parameter, the active or the idle-member in the correspondig vtable is called.
-    /// These are device-dependent again:
-    /// Device | active | idle
-    /// ------ | ------ | ----
-    /// RGB Keyboard | cmd_active_kb() means: start the device with a lot of kb-specific initializers (software controlled mode) | cmd_idle_kb() set the device with a lot of kb-specific initializers into the hardware controlled mode)
-    /// non RGB Keyboard | cmd_io_none() means: Do nothing | cmd_io_none() means: Do nothing
-    /// Mouse | cmd_active_mouse() similar to cmd_active_kb() | cmd_idle_mouse similar to cmd_idle_kb()
-    ///
-    /// - If either \a start() succeeded or the next following usb_tryreset(), it goes on, otherwise again a hard abort occurs.
+
     if(vt->start(kb, 0) && usb_tryreset(kb))
         goto fail_noinput;
     ///
@@ -659,11 +601,11 @@ int revertusb(usbdevice* kb){
 ///
 int _resetusb(usbdevice* kb, const char* file, int line){
     // Perform a USB reset
-    DELAY_LONG(kb);
+    DELAY_100MS();
     int res = os_resetusb(kb, file, line);
     if(res)
         return res;
-    DELAY_LONG(kb);
+    DELAY_100MS();
     // Re-initialize the device
     if(kb->vtable.start(kb, kb->active) != 0)
         return -1;
@@ -713,18 +655,13 @@ int usb_tryreset(usbdevice* kb){
     return -1;
 }
 
-///
-/// hwload_mode is defined in device.c
-///
-extern int hwload_mode;
-
 // Wrapper around the vtable write() function for error handling and recovery
 int _usbsend(usbdevice* kb, void* messages, size_t msg_len, int count, const char* file, int line){
     int total_sent = 0;
     for(int i = 0; i < count; i++){
         // Send each message via the OS function
         while(1){
-            DELAY_SHORT(kb);
+            kb->vtable.delay(kb, DELAY_SEND);
             queued_mutex_lock(mmutex(kb)); ///< Synchonization between macro and color information
             int res = kb->vtable.write(kb, messages + i * msg_len, msg_len, 0, file, line);
             queued_mutex_unlock(mmutex(kb));
@@ -735,10 +672,10 @@ int _usbsend(usbdevice* kb, void* messages, size_t msg_len, int count, const cha
                 break;
             }
             // Stop immediately if the program is shutting down or hardware load is set to tryonce
-            if(reset_stop || hwload_mode != 2)
+            if(reset_stop)
                 return 0;
             // Retry as long as the result is temporary failure
-            DELAY_LONG(kb);
+            DELAY_100MS();
         }
     }
     return total_sent;
@@ -752,7 +689,7 @@ int _usbrecv(usbdevice* kb, void* out_msg, size_t msg_len, uchar* in_msg, const
     for (int try = 0; try < 5; try++) {
         // Send the output message
         queued_mutex_lock(mmutex(kb)); ///< Synchonization between macro and color information
-        DELAY_SHORT(kb);
+        kb->vtable.delay(kb, DELAY_SEND);
         int res = kb->vtable.write(kb, out_msg, msg_len, 1, file, line);
         queued_mutex_unlock(mmutex(kb));
         if (res == 0)
@@ -761,21 +698,19 @@ int _usbrecv(usbdevice* kb, void* out_msg, size_t msg_len, uchar* in_msg, const
             // Retry on temporary failure
             if (reset_stop)
                 return 0;
-            DELAY_LONG(kb);
+            DELAY_100MS();
             continue;
         }
         // Wait for the response
-        if(!(kb->fwversion >= 0x120 || IS_V2_OVERRIDE(kb)) && kb->protocol != PROTO_BRAGI)
-            DELAY_MEDIUM(kb);
+        kb->vtable.delay(kb, DELAY_RECV);
         res = kb->vtable.read(kb, in_msg, msg_len, 0, file, line);
         if(res == 0)
             return 0;
         else if(res != -1)
             return res;
-        if(reset_stop || hwload_mode != 2)
+        if(reset_stop)
             return 0;
-        if(!(kb->fwversion >= 0x120 || IS_V2_OVERRIDE(kb)) && kb->protocol != PROTO_BRAGI)
-            DELAY_LONG(kb);
+        DELAY_100MS();
     }
     // Give up
     ckb_err_fn("Too many send/recv failures. Dropping.", file, line);
