@@ -40,7 +40,20 @@ public:
     QString usbModel, usbSerial;
     // Device information
     QStringList features;
-    QString pollrate;
+
+    enum pollrate_t {
+        POLLRATE_UNKNOWN = -1,
+        POLLRATE_8MS,
+        POLLRATE_4MS,
+        POLLRATE_2MS,
+        POLLRATE_1MS,
+        POLLRATE_05MS,
+        POLLRATE_025MS,
+        POLLRATE_01MS,
+        POLLRATE_COUNT,
+    };
+    pollrate_t pollrate, maxpollrate;
+
     bool monochrome;
     ushort productID;
     bool hwload;
@@ -89,13 +102,13 @@ public:
     void                            profiles(const QList<KbProfile*>& newProfiles)  { _needsSave = true; _profiles = newProfiles; }
     void                            appendProfile(KbProfile* newProfile)            { _needsSave = true; _profiles.append(newProfile); }
     inline int                      indexOf(KbProfile* profile)                     { return _profiles.indexOf(profile); }
-    inline KbProfile*               find(const QUuid& id)                           { foreach(KbProfile* profile, _profiles) { if(profile->id().guid == id) return profile; } return 0; }
+    inline KbProfile*               find(const QUuid& id)                           { foreach(KbProfile* profile, _profiles) { if(profile->id().guid == id) return profile; } return nullptr; }
 
     // Currently-selected mode
     inline KbMode*  currentMode()   { return _currentMode; }
-    inline KbLight* currentLight()  { return !_currentMode ? 0 : _currentMode->light(); }
-    inline KbBind*  currentBind()   { return !_currentMode ? 0 : _currentMode->bind(); }
-    inline KbPerf*  currentPerf()   { return !_currentMode ? 0 : _currentMode->perf(); }
+    inline KbLight* currentLight()  { return _currentMode ? _currentMode->light() : nullptr; }
+    inline KbBind*  currentBind()   { return _currentMode ? _currentMode->bind() : nullptr; }
+    inline KbPerf*  currentPerf()   { return _currentMode ? _currentMode->perf() : nullptr; }
 
     // Update selection
     void        setCurrentProfile(KbProfile* profile);
@@ -161,6 +174,7 @@ signals:
     void profileAdded();
     void profileRenamed();
     void batteryChanged(uint batteryLevel, BatteryStatus batteryStatus);
+    void batteryChangedLed(uint batteryLevel, BatteryStatus batteryStatus);
 
     void profileChanged();
     void profileAboutToChange();
