@@ -1,5 +1,6 @@
 #include "kbmanager.h"
 #include "idletimer.h"
+#include <limits>
 
 #ifndef Q_OS_MACOS
 QString devpath = "/dev/input/ckb%1";
@@ -7,8 +8,14 @@ QString devpath = "/dev/input/ckb%1";
 QString devpath = "/var/run/ckb%1";
 #endif
 
-QString KbManager::_guiVersion, KbManager::_daemonVersion = DAEMON_UNAVAILABLE_STR;
-KbManager* KbManager::_kbManager = 0;
+#ifdef DEBUG_IDLE_TIMER
+#define IDLE_TIMER_DURATION 5000
+#else
+#define IDLE_TIMER_DURATION (CkbSettings::get("Program/IdleTimerDuration", 5).toInt() * 60000)
+#endif
+
+CkbVersionNumber KbManager::_guiVersion, KbManager::_daemonVersion;
+KbManager* KbManager::_kbManager = nullptr;
 
 #ifdef USE_XCB_SCREENSAVER
 QTimer* KbManager::_idleTimer = nullptr;
@@ -20,26 +27,33 @@ void KbManager::setIdleTimer(bool enable){
         connect(_idleTimer, &QTimer::timeout, _kbManager, &KbManager::idleTimerTick);
     }
     if(enable)
-        _idleTimer->start(CkbSettings::get("Program/IdleTimerDuration", 5).toInt() * 60000);
+        _idleTimer->start(IDLE_TIMER_DURATION);
     else
         _idleTimer->stop();
 }
 void KbManager::idleTimerTick(){
     // Get user idle time
-    int idle = IdleTimer::getIdleTime();
-    int settingsidle = CkbSettings::get("Program/IdleTimerDuration", 5).toInt() * 60000;
-    int res = settingsidle - idle;
-    if(res > 0)
+    const int idle = IdleTimer::getIdleTime();
+    const int settingsidle = IDLE_TIMER_DURATION;
+    const int devidleres = settingsidle - getLastUsedDeviceIdleTime();
+    int systemidleres = settingsidle - idle;
+    if(systemidleres > 0 || devidleres > 0)
     {
+        if(devidleres > systemidleres)
+            systemidleres = devidleres;
+
         // Turn the lights back on (if applicable)
         foreach(Kb* kb, _devices){
-            kb->currentLight()->timerDimRestore();
+            // Check if the dimming state on this device could actually be restored
+            // If not, skip over it to avoid a failed brightness setting restoration with shared dimming
+            if(kb->currentLight()->timerDimRestore() == -1)
+                continue;
             if(KbLight::shareDimming() != -1)
                 break;
         }
 
         // Reschedule the timer if there's still time left
-        _idleTimer->start(res);
+        _idleTimer->start(systemidleres);
         return;
     }
     // Turn off all the lights
@@ -53,8 +67,9 @@ void KbManager::idleTimerTick(){
 }
 #endif
 
-void KbManager::init(QString guiVersion){
-    _guiVersion = guiVersion;
+void KbManager::init(const QString& guiVersion){
+    qRegisterMetaType<CkbVersionNumber>();
+    _guiVersion = CkbVersionNumber(guiVersion);
     if(_kbManager)
         return;
     _kbManager = new KbManager();
@@ -76,36 +91,38 @@ KbManager::KbManager(QObject *parent) : QObject(parent){
     // Set up the timers
     _eventTimer = new QTimer(this);
     _eventTimer->setTimerType(Qt::PreciseTimer);
+    _saveTimer = new QTimer(this);
+    _saveTimer->start(30 * 1000);
     _scanTimer = new QTimer(this);
     _scanTimer->start(1000);
-    connect(_scanTimer, SIGNAL(timeout()), this, SLOT(scanKeyboards()));
+    connect(_scanTimer, &QTimer::timeout, this, &KbManager::scanKeyboards);
+}
 
-    // Scan for keyboards immediately so they show up as soon as the GUI appears.
-    QTimer::singleShot(0,this,SLOT(scanKeyboards()));
+int KbManager::getLastUsedDeviceIdleTime(){
+    qint64 last = std::numeric_limits<qint64>::max();
+
+    for(const Kb* const dev : _devices){
+        const qint64 cur = dev->getDeviceIdleTime();
+        if(last > cur)
+            last = cur;
+    }
+
+    if(last > INT_MAX)
+        last = INT_MAX;
+    return (int)last;
 }
 
 void KbManager::fps(int framerate){
     QTimer* timer = eventTimer();
     if(!timer)
         return;
+    // Explicitly round the result to the nearest integer
+    // If we strip the decimal part, then we end up with 62.5 FPS instead of 60
+    const int target = roundf(1000.f / framerate);
     if(timer->isActive())
-        timer->setInterval(1000 / framerate);
+        timer->setInterval(target);
     else
-        timer->start(1000 / framerate);
-}
-
-float KbManager::parseVersionString(QString version){
-    // Remove extraneous info (anything after + or -, anything non-numeric)
-    QStringList dots = version.replace(QRegExp("\\+|-.+"), "").replace(QRegExp("[^\\d\\.]"), "").split(".");
-    float base = 1.f;
-    float res = 0.f;
-    // A number like "1.2.3" will result in 1.0203
-    // NB: will fail if a point version goes over 99 or if using more than two dots. floats can only reliably encode 7 decimal digits.
-    foreach(const QString& dot, dots){
-        res += dot.toFloat() * base;
-        base /= 100.f;
-    }
-    return res;
+        timer->start(target);
 }
 
 void KbManager::scanKeyboards(){
@@ -119,23 +136,21 @@ void KbManager::scanKeyboards(){
             delete kb;
         }
         _devices.clear();
-        if(_daemonVersion != DAEMON_UNAVAILABLE_STR){
-            _daemonVersion = DAEMON_UNAVAILABLE_STR;
+        if(!_daemonVersion.isNull()){
+            _daemonVersion = CkbVersionNumber();
             emit versionUpdated();
         }
         return;
     }
-    // Check daemon version
-    QFile version(rootdev + "/version");
-    QString vString;
-    if(version.open(QIODevice::ReadOnly)){
-        vString = QString::fromUtf8(version.readLine()).trimmed();
-        version.close();
-    } else
-        vString = DAEMON_UNAVAILABLE_STR;
-    if(_daemonVersion != vString){
-        _daemonVersion = vString;
-        emit versionUpdated();
+
+    if(_daemonVersion.isNull()){
+        // Check daemon version
+        QFile version(rootdev + "/version");
+        if(version.open(QIODevice::ReadOnly)){
+            _daemonVersion = CkbVersionNumber(QString::fromUtf8(version.readLine()).trimmed());
+            emit versionUpdated();
+            version.close();
+        }
     }
 
     // Scan connected devices
@@ -192,7 +207,29 @@ void KbManager::scanKeyboards(){
         // Load preferences and send signal
         emit kbConnected(kb);
         kb->load();
-        connect(_eventTimer, SIGNAL(timeout()), kb, SLOT(frameUpdate()));
-        connect(_scanTimer, SIGNAL(timeout()), kb, SLOT(autoSave()));
+        connect(_eventTimer, &QTimer::timeout, kb, &Kb::frameUpdate);
+        connect(_saveTimer, &QTimer::timeout, kb, &Kb::autoSave);
     }
 }
+
+void KbManager::brightnessScroll(int delta, Qt::Orientation orientation){
+    const bool up = delta > 0;
+    int dimming = KbLight::shareDimming();
+
+    // Only run this if shared dimming is enabled
+    if(KbLight::shareDimming() == -1)
+        return;
+
+    QSet<Kb*>::const_iterator i = _devices.begin();
+    if(_devices.empty() || i == _devices.end())
+        return;
+
+    dimming += (up ? -1 : 1);
+
+    if(dimming < 0)
+        dimming = 0;
+    if(dimming > KbLight::MAX_DIM)
+        dimming = KbLight::MAX_DIM;
+
+    (*i)->currentLight()->dimming(dimming);
+}
