@@ -45,7 +45,7 @@ void KbProfileDialog::profileList_reordered(){
 
 void KbProfileDialog::repopulate(){
     ui->profileList->clear();
-    QListWidgetItem* current = 0;
+    QListWidgetItem* current = nullptr;
     foreach(KbProfile* profile, device->profiles()){
         QListWidgetItem* item = new QListWidgetItem(QIcon((profile == device->hwProfile()) ? ":/img/icon_profile_hardware.png" : ":/img/icon_profile.png"), profile->name(), ui->profileList);
         item->setData(GUID, profile->id().guid);
@@ -189,7 +189,7 @@ void KbProfileDialog::on_profileList_customContextMenuRequested(const QPoint &po
     repopulate();
 }
 
-void KbProfileDialog::extractedFileCleanup(QStringList extracted){
+void KbProfileDialog::extractedFileCleanup(const QStringList& extracted){
     for(int i = 0; i < extracted.count(); i++){
         // Delete extracted files
         QFile fdel(extracted.at(i));
@@ -317,7 +317,7 @@ void KbProfileDialog::on_exportButton_clicked(){
         QMessageBox::information(this, tr("Export Successful"), tr("Selected profiles have been exported successfully."), QMessageBox::Ok);
 }
 
-bool KbProfileDialog::verifyHash(QString file){
+bool KbProfileDialog::verifyHash(const QString& file){
     QFile iniFile(file);
     QFile hashFile(file + ".s256");
     if(iniFile.open(QFile::ReadOnly) && hashFile.open(QFile::ReadOnly)){
@@ -335,7 +335,7 @@ bool KbProfileDialog::verifyHash(QString file){
     return false;
 }
 
-void KbProfileDialog::importCleanup(QStringList extracted, QList<QPair<CkbExternalSettings*, QString>> profileptrs){
+void KbProfileDialog::importCleanup(const QStringList& extracted, const QList<QPair<CkbExternalSettings*, QString>>& profileptrs){
     // Destroy the open CkbExternalSettings objects
     for(int i = 0; i < profileptrs.count(); i++)
         delete profileptrs.at(i).first;
@@ -356,7 +356,7 @@ void KbProfileDialog::on_importButton_clicked(){
     QStringList filenames = dialog.selectedFiles();
 
     // Pick only the first filename
-    QString filename = filenames.at(0);
+    const QString& filename = filenames.at(0);
     if(!filename.endsWith(".ckb")){
         QMessageBox::warning(this, tr("Error"), tr("Selected file is not a valid profile."), QMessageBox::Ok);
         return;
@@ -438,7 +438,7 @@ void KbProfileDialog::on_importButton_clicked(){
     int pkgVerifiedCount = 0;
 
     for(int i = 0; i < extracted.count(); i++){
-        QString tmpFile = extracted.at(i);
+        const QString& tmpFile = extracted.at(i);
         if(tmpFile.endsWith("ini")){
             pkgProfileCount++;
             if(verifyHash(tmpFile)){
@@ -482,7 +482,7 @@ void KbProfileDialog::on_importButton_clicked(){
         //QUuid current = guid.trimmed();
         QUuid guid = sptr->childGroups().first().trimmed();
         // Messy, shhhh
-        QString profname = profilestr.at(i);
+        const QString& profname = profilestr.at(i);
         KbProfile* profilematch = nullptr;
         foreach(KbProfile* profile, device->profiles()){
 
