@@ -33,11 +33,9 @@ public:
     void updateFwButton();
     void setTabBarEnabled(const bool e);
 
-public slots:
-    // Show a tab
-    void showFirstTab();
-    void showLastTab();
+    void showDeviceTab();
 
+public slots:
     // Display firmware update dialog
     inline void showFwUpdate()          { on_fwUpdButton_clicked(); }
 
@@ -49,7 +47,11 @@ private:
 
     const static int GUID = Qt::UserRole;
     const static int NEW_FLAG = Qt::UserRole + 1;
-    int getPollRateBoxIdx(QString poll);
+    inline int getPollRateBoxIdx(Kb::pollrate_t poll){
+        if(poll == Kb::POLLRATE_UNKNOWN)
+            return 0;
+        return ui->pollRateBox->count() - poll - 1;
+    }
 
     KbMode* prevmode;
     void openEventMgr(KbMode* mode);
@@ -66,14 +68,16 @@ private slots:
     void modeChanged();
     void currentSelectionChanged(const QModelIndex& current, const QModelIndex& previous);
     void on_modesList_customContextMenuRequested(const QPoint &pos);
+    void batteryTrayBox_stateChanged(int state);
 
     void devUpdate();
+    void updateBattery(uint battery, BatteryStatus charging);
     void on_hwSaveButton_clicked();
     void on_tabWidget_currentChanged(int index);
     void on_fwUpdButton_clicked();
     void on_layoutBox_activated(int index);
-    void switchToProfile(QString profile);
-    void switchToMode(QString mode);
+    void switchToProfile(const QString& profile);
+    void switchToMode(const QString& mode);
     void on_pollRateBox_currentIndexChanged(const QString& arg1);
     void switchToModeByFocus(XWindowInfo win);
     void on_modesList_doubleClicked(const QModelIndex& index);
