@@ -25,7 +25,7 @@ class MainWindow : public QMainWindow
     Q_OBJECT
 
 public:
-    explicit MainWindow(QWidget *parent = 0);
+    explicit MainWindow(const bool silent, QWidget *parent = nullptr);
     ~MainWindow();
 
     static MainWindow* mainWindow;
